@@ -26,7 +26,7 @@ import gtk
 
 RSS_NAME = 'ReSiStance'
 RSS_COMPACT_NAME = 'resistance'
-RSS_VERSION = '0.9.1'
+RSS_VERSION = '0.9.2'
 RSS_DESCRIPTION = 'ReSiStance is a RSS feeds reader'
 RSS_URL = 'http://www.igalia.com'
 
@@ -78,3 +78,7 @@ STATE_SUFFIX = "state/com.google/"
 URL_EDIT = GOOGLE_READER_API_URL + 'edit-tag?client=-'
 URL_SUBSCRIPTION_EDIT = GOOGLE_READER_API_URL + 'subscription/edit'
 GOOGLE_READER_ICON_FILE = os.path.join(APP_DIR, 'prism-google-reader.png')
+
+# Labels
+ALL_LABEL_ID = 0
+RSS_LABEL_DB_FILE = os.path.join(RSS_CONF_FOLDER, '%s' % 'labels.db')
