@@ -120,6 +120,10 @@ class ReSiStanceFeedSummary(dict):
         self.unread = len([entry for entry in feed_data.entries if entry.read == False])
 
 class FeedManager(gobject.GObject):
+    __gsignals__ = {
+        "feed-added": (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (int, )),
+        "label-created": (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (int, )),
+        }
 
     LOW_PRIORITY = 10
     DEFAULT_PRIORITY = 0
@@ -131,20 +135,25 @@ class FeedManager(gobject.GObject):
         self._settings = settings
         self._feed_data_dict = {}
         self._feeds_summary = {}
+        self._feeds_summary_loaded = False
         self._conn_manager = conn_manager
         self._conn_manager.connect('connection-changed', self._on_connection_changed)
         self._google_reader_auth = None
         self._google_reader_auth_token = None
         self._google_reader_expires = None
         self._async_worker = None
+        self._label_dict = {}
+        self._label_dict_loaded = False
 
     def stop(self):
         # Cancel all pending operations
-        self._async_worker.stop()
+        if self._async_worker:
+            self._async_worker.stop()
 
     def halt(self):
         # Wait till all the workers are done with their tasks
-        self._async_worker.halt()
+        if self._async_worker:
+            self._async_worker.halt()
 
     def requireWorker(f):
         '''Function decorator to ensure that a valid async worker
@@ -203,6 +212,8 @@ class FeedManager(gobject.GObject):
         else:
             pixbuf = self._get_favicon_sync(new_feed_data.href)
 
+        self.emit('feed-added', new_feed_id)
+
         return pixbuf, new_feed_data
 
     @requireWorker
@@ -492,6 +503,7 @@ class FeedManager(gobject.GObject):
             return None
 
         urls = []
+        label_feeds = {}
         try:
             # Get subscriptions
             subs_request = urllib2.Request(constants.URL_SUBSCRIPTION_LIST, headers = auth_headers)
@@ -514,12 +526,19 @@ class FeedManager(gobject.GObject):
                 # <string name="id">webfeed/someidreturnedbygoogle</string>
                 # <string name="id">user/someuserid/label/SomeLabel</string>
                 if feed_url.startswith('feed/'):
-                    print 'Importing: ' + feed_url[5:]
                     urls.append(feed_url[5:])
+
+                for element in node.getElementsByTagName('list'):
+                    if element.getAttribute('name') == 'categories':
+                        for category in element.childNodes:
+                            label_name = category.childNodes[1].firstChild.data
+                            if label_name not in label_feeds:
+                                label_feeds[label_name] = []
+                            label_feeds[label_name].append(feed_url[5:])
         except:
             pass
 
-        return urls
+        return urls, label_feeds
 
     @requireWorker
     def save(self, callback=None, data=None):
@@ -530,11 +549,19 @@ class FeedManager(gobject.GObject):
 
     def save_sync(self):
 
-        # First save summary, then feed data
-        try:
-            self._save_feeds_summary()
-        except:
-            print 'Unable to save feeds summary'
+        # Save labels, summaries and feed data
+        if self._label_dict_loaded:
+            try:
+                self._save_labels()
+            except:
+                print 'Unable to save label info'
+
+        # Summaries may not have been loaded
+        if self._feeds_summary_loaded:
+            try:
+                self._save_feeds_summary()
+            except:
+                print 'Unable to save feeds summary'
 
         for feed in self._feed_data_dict.values():
             file_name = os.path.join(constants.RSS_DB_DIR, str(get_feed_id(feed)))
@@ -756,6 +783,9 @@ class FeedManager(gobject.GObject):
     def get_feed_summary(self, feed_id):
         return self._feeds_summary[feed_id]
 
+    def get_feed_summaries(self):
+        return self._feeds_summary.values()
+
     def get_feed_data(self, feed_id):
         try:
             return self._feed_data_dict[feed_id]
@@ -785,17 +815,29 @@ class FeedManager(gobject.GObject):
             self._feeds_summary[feed_id] = self._generate_summary(feed_data, feed_data.sync, feed_data.visits)
             self._feed_data_dict[feed_id] = ReSiStanceFeedDict(feed_data)
 
+        if self._feeds_summary:
+            self._feeds_summary_loaded = True
+
     def _load_feeds_summary_in_thread(self):
         try:
             db_file = open(constants.RSS_DB_SUMMARY_FILE, 'r')
             self._feeds_summary = cPickle.load(db_file)
+            self._feeds_summary_loaded = True
             db_file.close()
         except IOError, e:
+            # IOError 2 means "file not found", so we're moving from
+            # an old version of ReSiStance that does not use summaries.
             if e.errno != 2:
                 print e
                 return None
-            # IOError 2 means "file not found", so we're moving from
-            # an old version of ReSiStance that does not use summaries.
+
+            # There is no summary nor old storage, this means that
+            # this is the first execution. self._feeds_summary_loaded
+            # must be True to allow new feeds to be stored
+            self._feeds_summary_loaded = True
+            if not os.path.isfile(constants.OLD_RSS_DB_FILE):
+                return None
+
             self._create_summary_from_old_storage()
 
             # No need to keep the old storage. Try to save summaries
@@ -820,3 +862,62 @@ class FeedManager(gobject.GObject):
         db_file = open(constants.RSS_DB_SUMMARY_FILE, 'w')
         cPickle.dump(self._feeds_summary, db_file)
         db_file.close()
+
+    # LABELS
+    def get_label_list(self):
+        return zip(self._label_dict.keys(), [label_data[0] for label_data in self._label_dict.values()])
+
+    def get_label_name(self, label_id):
+        return self._label_dict[label_id][0]
+
+    def get_feeds_for_label(self, label_id):
+        return self._label_dict[label_id][1]
+
+    def feed_has_label(self, feed_id, label_id):
+        if label_id == constants.ALL_LABEL_ID:
+            return True
+        return feed_id in self._label_dict[label_id][1]
+
+    def remove_label(self, label_id):
+        del self._label_dict[label_id]
+
+    def create_label(self, label_name):
+        label_id = hash(label_name)
+        if label_id in self._label_dict:
+            return 0
+
+        self._label_dict[label_id] = [label_name, []]
+        self.emit('label-created', label_id)
+        return label_id
+
+    def add_feeds_to_label(self, feed_ids, label_id):
+        self._label_dict[label_id][1] += feed_ids
+
+    def remove_feeds_from_label(self, feed_ids, label_id):
+        for feed_id in feed_ids:
+            self._label_dict[label_id][1].remove(feed_id)
+
+    @requireWorker
+    def load_labels(self, callback, data=None):
+        load_item = AsyncItem(self._load_labels_in_thread, (),
+                              callback, (data,))
+        self._async_worker.add_task((self.HIGH_PRIORITY, load_item))
+
+    def _load_labels_in_thread(self):
+        try:
+            db_file = open(constants.RSS_LABEL_DB_FILE, 'r')
+            self._label_dict = cPickle.load(db_file)
+            db_file.close()
+            loaded = True
+        except IOError:
+            print 'No labels file or unable to read it'
+            loaded = False
+        finally:
+            self._label_dict_loaded = True
+
+        return loaded
+
+    def _save_labels(self):
+        db_file = open(constants.RSS_LABEL_DB_FILE, 'w')
+        cPickle.dump(self._label_dict, db_file)
+        db_file.close()
