@@ -24,8 +24,10 @@ import glib
 import gtk
 import htmlentitydefs
 import re
+from gobject import markup_escape_text
 from time import localtime, strftime
 from os import getenv
+from xml.dom.minidom import parseString
 
 # SYSTEM COLORS
 color_style = gtk.rc_get_style_by_paths(gtk.settings_get_default() , 'GtkButton', 'osso-logical-colors', gtk.Button)
@@ -57,19 +59,43 @@ def unescape(text):
     return re.sub("&#?\w+;", fixup, text)
 
 def get_author_from_item(item):
-    if 'publisher_detail' in item:
-        author = item.publisher_detail.name
+    from_title = False
+
+    # I found some feeds with author_detail but without name,
+    # email..., just with a 'value' key
+    if 'author_detail' in item and 'name' in item.author_detail:
+        author = item.author_detail.name
     elif 'author' in item:
         author = item.author
+    elif 'publisher_detail' in item and 'name' in item.publisher_detail:
+        author = item.publisher_detail.name
     else:
+        from_title = True
         words = item.title.rsplit(":")
         if len(words) > 1:
             author = words[0].lstrip()
         else:
             author = ''
 
+    if not from_title:
+        try:
+            doc = parseString(author)
+            author = doc.documentElement.firstChild.data
+        except:
+            pass
+
     return author
 
+def get_title_from_item(item):
+    words = item.title.rsplit(":")
+    author = get_author_from_item(item)
+    if len(words) > 1 and author == words[0].lstrip():
+        title = words[1].lstrip()
+    else:
+        title = item.title
+
+    return markup_escape_text(unescape(title))
+
 def get_feed_title_markup(title):
     return '<span foreground="%s">%s</span>' % \
         (DEFAULT_TEXT_COLOR, glib.markup_escape_text(unescape(title)))
